-- Sonic & Knuckles - Blue Sphere Special Stage logic
-- Script by Justburner @ 2019/05/01
-- Version 5 @ 2019/09/05

freeslot(
	"MT_SNKSPHERE_BLUE",
	"MT_SNKSPHERE_RED",
	"MT_SNKSPHERE_RBLUE",
	"MT_SNKSPHERE_WALL",
	"MT_SNKSPHERE_RWALL",
	"MT_SNKSPHERE_LWALL",
	"MT_SNKSPHERE_JUMP",
	"sfx_bumper",
	"sfx_wrgwae"
)

-- Check for red spheres
--  if blue: turns red
--  if red:  kills player
local function LA_RedSphereCheck(special, toucher)
	if special.state == S_SNKSPHERE_BLUE
		special.state = S_SNKSPHERE_BLUE_TOUCH
	end
	if special.state == S_SNKSPHERE_RBLUE
		special.state = S_SNKSPHERE_RBLUE_TOUCH
	end
	if special.state == S_SNKSPHERE_RED or special.state == S_SNKSPHERE_RRED
		if (special.spawnpoint ~= nil and special.spawnpoint.angle == 999)
			S_StartSound(toucher, sfx_wrgwae)
			if (toucher.player ~= nil)
				print(toucher.player.name .. " got knuckled!")
			end
		end
		P_DamageMobj(toucher, special, special, 10000)
	end
	return true
end

-- Play a bumping sound
local function LA_BumperSphere(thing, tmthing)
	server.SnKBlueSphere_on = true
	if tmthing.player ~= nil
		-- see if it went over / under
		if (tmthing.z > thing.z + thing.height)
			return -- overhead
		end
		if (tmthing.z + tmthing.height < thing.z)
			return -- underneath
		end
		-- Play the sound
		if tmthing.player.SnKBlueSphere_MB == nil
			S_StartSound(thing, sfx_bumper)
			tmthing.player.SnKBlueSphere_MB = true
			tmthing.player.SnKBlueSphere_MBC = 2 -- Cooldown time
		end
	end
	return
end

-- Initialize artificial spring action (Directional)
local function LA_SpringSphereDirectional(special, toucher)
	if (special.state == S_SNKSPHERE_JUMP and toucher.player ~= nil and special.spawnpoint ~= nil)
		server.SnKBlueSphere_on = true
		local player = toucher.player
		special.state = S_SNKSPHERE_JUMP_ACT
		local anglex = special.spawnpoint.angle
		local spheres = (anglex / 360) + 1
		-- has to fly 64u per sphere
		local calcVel = spheres * 64 * FRACUNIT / 33
		if player.SnKBlueSphere_cnt == nil
			player.SnKBlueSphere_ms = P_AproxDistance(toucher.momx, toucher.momy) * 4/5 -- writing 0.8 won't work
			player.SnKBlueSphere_ms = max(16*FRACUNIT, player.SnKBlueSphere_ms) -- speed of 16u atleast
		end
		toucher.angle = special.angle
		player.SnKBlueSphere_cnt = 35 -- expected 1 sec fly time (33 real)
		player.SnKBlueSphere_px = special.x
		player.SnKBlueSphere_py = special.y
		player.SnKBlueSphere_pz = special.z + 26*FRACUNIT
		player.SnKBlueSphere_vx = FixedMul(cos(special.angle), calcVel)
		player.SnKBlueSphere_vy = FixedMul(sin(special.angle), calcVel)
		player.SnKBlueSphere_vz = 16*FRACUNIT 
		player.SnKBlueSphere_ma = special.angle
	end
	return true
end

-- Simulate artificial spring action
addHook("ThinkFrame", do
	if server.SnKBlueSphere_on ~= nil
		for player in players.iterate
			if player.SnKBlueSphere_cnt ~= nil
				player.mo.momx = 1
				player.mo.momy = 1
				player.mo.momz = 1
				P_TeleportMove(player.mo, player.SnKBlueSphere_px+player.SnKBlueSphere_vx, player.SnKBlueSphere_py+player.SnKBlueSphere_vy, player.SnKBlueSphere_pz+player.SnKBlueSphere_vz)
				player.SnKBlueSphere_px = player.SnKBlueSphere_px + player.SnKBlueSphere_vx
				player.SnKBlueSphere_py = player.SnKBlueSphere_py + player.SnKBlueSphere_vy
				player.SnKBlueSphere_pz = player.SnKBlueSphere_pz + player.SnKBlueSphere_vz
				player.SnKBlueSphere_vz = player.SnKBlueSphere_vz - FRACUNIT -- artificial gravity of 1.0
				player.SnKBlueSphere_cnt = player.SnKBlueSphere_cnt - 1
				if player.SnKBlueSphere_cnt <= 0
					player.SnKBlueSphere_cnt = nil
					player.mo.momx = FixedMul(cos(player.SnKBlueSphere_ma), player.SnKBlueSphere_ms)
					player.mo.momy = FixedMul(sin(player.SnKBlueSphere_ma), player.SnKBlueSphere_ms)
					player.mo.momz = player.SnKBlueSphere_vz
				else
					player.mo.momx = player.SnKBlueSphere_vx
					player.mo.momy = player.SnKBlueSphere_vy
					player.mo.momz = player.SnKBlueSphere_vz
				end
			end
			-- Check if player is free to move and actual movement to prevent constant playback of the bumper
			if player.mo ~= nil then
				if player.SnKBlueSphere_LX == nil
					player.SnKBlueSphere_LX = player.mo.x
					player.SnKBlueSphere_LY = player.mo.y
				end
				local b,m = P_CheckPosition(player.mo, player.mo.x, player.mo.y)
				if player.SnKBlueSphere_MBC ~= nil
					-- Cooldown
				    if player.SnKBlueSphere_MBC > 0
						player.SnKBlueSphere_MBC = player.SnKBlueSphere_MBC - 1
					else
						player.SnKBlueSphere_MBC = nil
					end
				else
					if b == true and P_AproxDistance(player.mo.x-player.SnKBlueSphere_LX, player.mo.y-player.SnKBlueSphere_LY) >= 5*FRACUNIT
						player.SnKBlueSphere_MB = nil
					end
				end
				player.SnKBlueSphere_LX = player.mo.x
				player.SnKBlueSphere_LY = player.mo.y
			end
		end
		if gamecomplete
			server.SnKBlueSphere_on = nil
		end
	end
end)

-- Hooks
addHook("TouchSpecial", LA_RedSphereCheck, MT_SNKSPHERE_BLUE)
addHook("TouchSpecial", LA_RedSphereCheck, MT_SNKSPHERE_RED)
addHook("TouchSpecial", LA_RedSphereCheck, MT_SNKSPHERE_RBLUE)
addHook("MobjCollide", LA_BumperSphere, MT_SNKSPHERE_WALL)
addHook("TouchSpecial", LA_SpringSphereDirectional, MT_SNKSPHERE_JUMP)
